@@ -5,11 +5,11 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2017, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2021, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
- * are also available at https://curl.haxx.se/docs/copyright.html.
+ * are also available at https://curl.se/docs/copyright.html.
  *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
@@ -24,18 +24,28 @@
 #include "speedcheck.h"
 #include "urldata.h"
 
+static CURL *easy;
+
 static CURLcode unit_setup(void)
 {
-  return CURLE_OK;
+  int res = CURLE_OK;
+
+  global_init(CURL_GLOBAL_ALL);
+  easy = curl_easy_init();
+  if(!easy) {
+    curl_global_cleanup();
+    return CURLE_OUT_OF_MEMORY;
+  }
+  return res;
 }
 
 static void unit_stop(void)
 {
-
+  curl_easy_cleanup(easy);
+  curl_global_cleanup();
 }
 
-static int runawhile(struct Curl_easy *easy,
-                     long time_limit,
+static int runawhile(long time_limit,
                      long speed_limit,
                      curl_off_t speed,
                      int dec)
@@ -66,25 +76,16 @@ static int runawhile(struct Curl_easy *easy,
 }
 
 UNITTEST_START
-{
-  struct Curl_easy *easy = curl_easy_init();
-  abort_unless(easy, "out of memory");
-
-  fail_unless(runawhile(easy, 41, 41, 40, 0) == 41,
+  fail_unless(runawhile(41, 41, 40, 0) == 41,
               "wrong low speed timeout");
-  fail_unless(runawhile(easy, 21, 21, 20, 0) == 21,
+  fail_unless(runawhile(21, 21, 20, 0) == 21,
               "wrong low speed timeout");
-  fail_unless(runawhile(easy, 60, 60, 40, 0) == 60,
+  fail_unless(runawhile(60, 60, 40, 0) == 60,
               "wrong log speed timeout");
-  fail_unless(runawhile(easy, 50, 50, 40, 0) == 50,
+  fail_unless(runawhile(50, 50, 40, 0) == 50,
               "wrong log speed timeout");
-  fail_unless(runawhile(easy, 40, 40, 40, 0) == 99,
+  fail_unless(runawhile(40, 40, 40, 0) == 99,
               "should not time out");
-  fail_unless(runawhile(easy, 10, 50, 100, 2) == 36,
+  fail_unless(runawhile(10, 50, 100, 2) == 36,
               "bad timeout");
-
-  curl_easy_cleanup(easy);
-
-  return 0;
-}
 UNITTEST_STOP
